package  com.activeTuts.fsm
{
	import flash.display.Sprite;
	import flash.events.Event;

	public class Car extends Sprite 
	{		
		///CAR STATES
		//engine off state
		//engine on state
		//drive state
		//no gas state
		
		public static const ONE_SIXTH_SECONDS:Number = 1 / 6; //6 times per second
		
		public static const IDLE_FUEL_CONSUMPTION:Number = .0055;
		public static const DRIVE_FUEL_CONSUMPTION:Number = .011;
		
		private var _engineOffState:IState;
		private var _engineOnState:IState;
		private var _engineDriveForwardState:IState;
		private var _engineOutOfFuelState:IState;
		
		private var _fuelCapacity:Number = 1;
		private var _fuelSupply:Number = _fuelCapacity; //starting on a full tank (in gallons)
		
		private var _engineTimer:Number = 0;
		
		private var _currentState:IState;
		
		public function Car () 
		{
			init ();
		}
		
		private function init ():void
		{
			initializeStates ();
		}
		
		private function initializeStates ():void
		{
			_engineOffState = new EngineOff (this);
			_engineOnState = new EngineOn (this);
			_engineDriveForwardState = new EngineDriveForward (this);
			_engineOutOfFuelState = new EngineOutOfFuel (this);
			
			_currentState = _engineOffState;
		}
		
		public function update ($tick:Number):void 
		{
			_currentState.update ($tick);
		}
		
		///car functions
		public function turnKeyOn (e:Event = null):void
		{
			_currentState.turnKeyOn ();
		}
		
		public function turnKeyOff (e:Event = null):void
		{
			_currentState.turnKeyOff ();
		}
		
		public function driveForward (e:Event = null):void
		{
			_currentState.driveForward ();
		}
		
		public function reFuel (e:Event = null):void
		{
			_currentState.reFuel ();
		}
		
		public function consumeFuel ($consumption:Number):void
		{
			
			if ((_fuelSupply -= $consumption) <= 0)
			{
				_fuelSupply = 0;
				print ("the engine has stopped, no more fuel to run...");
				
				changeState (_engineOutOfFuelState);
			}
		}
		
		public function refillWithFuel ():Number
		{
			var neededSupply:Number = _fuelCapacity - _fuelSupply;
			_fuelSupply += neededSupply;
			return neededSupply;
		}
		
		public function hasFullTank ():Boolean
		{
			var fullTank:Boolean = _fuelCapacity == _fuelSupply ? true : false;
			if (fullTank) print ("no need to refuel right now, the tank is full...");
			return fullTank;
		}
		
		public function getEngineOffState ():IState { return _engineOffState; } //explicit, you know you're calling a method
		
		public function getEngineOnState ():IState { return _engineOnState; }
		
		public function getEngineOutOfFuelState ():IState { return _engineOutOfFuelState; }
		
		public function getEngineDriveForwardState ():IState { return _engineDriveForwardState; }
		
		public function changeState ($state:IState):void 
		{
			_currentState = $state; 
		}
		
		public function get engineTimer ():Number { return _engineTimer; } //implicit, as if you're accessing a public variable
		
		public function set engineTimer ($value:Number):void { _engineTimer = $value; }		
		
		public function print ($text:String):void
		{
			trace ($text);
		}
		
		override public function toString ():String 
		{
			return 'The car is currently ' + _currentState + ' with a fuel amount of ' + _fuelSupply +  ' gallon(s).';
		}
	}
}